import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

public class Aquarium {
	public static void main(String[] args) {

		PrintStream out = System.out;

	      // Request document building without validation
     		SAXBuilder builder = new SAXBuilder(false);
		try {
   	  		Document doc = builder.build(new File("aquarium.xml"));
			Aquarium aq = new Aquarium();
			aq.printContents(doc, out);		
			

			// make the window
			JFrame frame = new JFrame("The Aquarium");
			JPanel contents = aq.makeContents(aq, doc);
			frame.getContentPane().add(contents);
	
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			frame.pack();
			frame.setVisible(true);

		} catch (JDOMException e) { 
			out.print("\t *** Error opening xml document\n");
			e.printStackTrace();
		}

	} // end of main()

	private void printContents(Document doc, PrintStream out) {
			// Get the root element (<aquarium>)
	      	Element root = doc.getRootElement();
			// Print contents info
			java.util.List contents = root.getChildren();
			out.println("The aquarium holds "+ contents.size() +" items.\n");
			java.util.List fish = root.getChildren("fish");
			out.println("Of these, "+ fish.size() +" are fish:\n");
			Iterator f= fish.iterator();
			while (f.hasNext()) {
				Element thefish = (Element) f.next();
				out.print("\t A " + thefish.getChild("type").getText() +
					 " named " + thefish.getChild("name")
								.getChild("value")
								.getText() + "\n");
			}
	} // end of printContents()

	class TickListener implements ActionListener {
		Document doc;
		DisplayArea display;
		public TickListener(Document doc, DisplayArea display) { 
			this.doc = doc;
			this.display = display;
		}

            public final void actionPerformed(ActionEvent e) {
    			Element root = doc.getRootElement();
			// move all the fish
			java.util.List fish = root.getChildren("fish");
			Iterator f= fish.iterator();
			while (f.hasNext()) {
				Element thefish = (Element) f.next();
				String oldx = thefish.getChild("graphics").getChild("position").getAttributeValue("x");
				String newx;
				if (Integer.parseInt(oldx) >= -20) {
					newx = String.valueOf(Integer.parseInt(oldx) - 10);
				} else {
					newx = "520";
				}
				thefish.getChild("graphics").getChild("position").setAttribute("x",newx);
			}
			// update the display
			display.repaint();
	      }
	} // end of TickListener class

	private JPanel makeContents(Aquarium aq, Document doc) {
		JPanel pane = new JPanel();
		pane.setBorder(BorderFactory.createEmptyBorder( 0,20,25,20 ));
		pane.setLayout(new BoxLayout(pane,BoxLayout.Y_AXIS));
		pane.add(new JLabel("<html><h1>The Aquarium</html></h1>"));
		JButton tickbutton = new JButton("Tick!");
		pane.add(tickbutton);
		DisplayArea display = new DisplayArea(doc);
		pane.add(display);
		TickListener ticker = new TickListener(doc,display);
		tickbutton.addActionListener(ticker);
		return pane;
	}



	public void tick(Document doc) {
		// Get the root element (<aquarium>)
	      Element root = doc.getRootElement();
		
		// move all the fish
		java.util.List fish = root.getChildren("fish");
		Iterator f= fish.iterator();
		while (f.hasNext()) {
			Element thefish = (Element) f.next();
		//	thefish.getChild("graphics").getChild("position").setAttributeValue("x",
		}
	}
	
	class DisplayArea extends JPanel {
	    Document doc;
	    Dimension preferredSize = new Dimension(500,500);
	
	    public DisplayArea(Document doc) {
	        this.doc = doc;
	
	        Border raisedBevel = BorderFactory.createRaisedBevelBorder();
	        Border loweredBevel = BorderFactory.createLoweredBevelBorder();
	        Border compound = BorderFactory.createCompoundBorder
	                              (raisedBevel, loweredBevel);
	        setBorder(compound);
	
	        addMouseListener(new MouseAdapter() {
	            public void mousePressed(MouseEvent e) {
			// do stuff
	                repaint();
	            }
	        });
	    }

	    public void paintComponent(Graphics g) {
	        super.paintComponent(g);  //paint background
	
	        //If user has chosen a point, paint a tiny rectangle on top.
	        if (doc != null) {
			// Get the root element (<aquarium>)
	      	Element root = doc.getRootElement();
			java.util.List fish = root.getChildren("fish");

			// draw each fish
			Iterator f= fish.iterator();
			while (f.hasNext()) {
				Element thefish = (Element) f.next();
				Element pos = thefish.getChild("graphics").getChild("position");
				int size = Integer.parseInt(thefish.getChild("size").getText());
				int x = Integer.parseInt(pos.getAttributeValue("x"));
				int y = Integer.parseInt(pos.getAttributeValue("y"));
				int width=7*size;
				int height=4*size;
				int xtl=x-width/2, ytl=y-height/2;

				final Color red = Color.red;
	   		      final Color white = Color.white;
        			GradientPaint redtowhite = new GradientPaint(xtl,ytl,red,xtl+3*width/2, ytl,white);
				final Color orange = Color.orange;
        			GradientPaint orangetowhite = new GradientPaint(xtl,ytl,orange,xtl+3*width/2, ytl,white);
        
        			Graphics2D g2 = (Graphics2D) g;
			      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				if (thefish.getChild("type").getText().compareTo("Parrotfish")==0) {
					g2.setPaint(redtowhite);
				} else {
					g2.setPaint(orangetowhite);	
				}

				// draw body
				g2.fill (new Ellipse2D.Double(xtl, ytl, width, height));
				// draw tail
				int x3Points[] = {x+width/4, x+width, x+width};
				int y3Points[] = {y, y-height/2, y+height/2};

				GeneralPath filledPolygon = new
  		            GeneralPath(GeneralPath.WIND_EVEN_ODD, x3Points.length);
				filledPolygon.moveTo(x3Points[0],y3Points[0]);
				for (int index = 1; index < x3Points.length; index++) 	{
					filledPolygon.lineTo(x3Points[index],
                             y3Points[index]);
				};
				filledPolygon.closePath();
				g2.fill(filledPolygon);

				// draw name
				g2.setPaint(Color.black);
				g2.fill (new Ellipse2D.Double(x, y, 5, 5));
				g2.drawString(thefish.getChild("name").getChild("value").getText(),
						x - Integer.parseInt(
							thefish.getChild("name").getChild("graphics").getChild("offset").getAttributeValue("x")),
						y + Integer.parseInt(
							thefish.getChild("name").getChild("graphics").getChild("offset").getAttributeValue("y")));		
			}
	        }
	    }
	    public Dimension getPreferredSize() {
	        return preferredSize;
	    }

	} // end DisplayArea class

} // end of Aquarium class